@@ -5,7 +5,7 @@
     'name_bg_BG': 'Инициативи и възможности за продажба',
     'name_de_DE': 'Verkauf Interessenten und Chancen',
     'name_fr_FR': 'Contacts commerciaux et opportunités de ventes',
-    'version': '2.0.3',
+    'version': '2.2.0',
     'author': 'Sharoon Thomas, Openlabs',
     'email': 'info@openlabs.co.in',
     'website': 'http://www.openlabs.co.in/',
@@ -49,8 +49,13 @@
         'party.xml',
     ],
     'translation': [
-        'bg_BG.csv',
-        'de_DE.csv',
-        'fr_FR.csv',
+        'locale/bg_BG.po',
+        'locale/cs_CZ.po',
+        'locale/de_DE.po',
+        'locale/es_CO.po',
+        'locale/es_ES.po',
+        'locale/fr_FR.po',
+        'locale/nl_NL.po',
+        'locale/ru_RU.po',
     ],
 }
