@@ -1,7 +1,6 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
 "Sales extension for managing leads and opportunities"
-from __future__ import with_statement
 import datetime
 import time
 from trytond.model import ModelView, ModelSQL, ModelWorkflow, fields
@@ -9,6 +8,7 @@ from trytond.wizard import Wizard
 from trytond.backend import FIELDS
 from trytond.pyson import Equal, Eval, Not, In, If, Get, PYSONEncoder
 from trytond.transaction import Transaction
+from trytond.pool import Pool
 
 STATES = [
     ('lead', 'Lead'),
@@ -18,11 +18,13 @@ STATES = [
     ('lost', 'Lost'),
 ]
 _STATES_START = {
-    'readonly': Not(Equal(Eval('state'), 'lead')),
-}
+    'readonly': Eval('state') != 'lead',
+    }
+_DEPENDS_START = ['state']
 _STATES_STOP = {
     'readonly': In(Eval('state'), ['converted', 'lost', 'cancelled']),
 }
+_DEPENDS_STOP = ['state']
 
 
 class SaleOpportunity(ModelWorkflow, ModelSQL, ModelView):
@@ -33,16 +35,16 @@ class SaleOpportunity(ModelWorkflow, ModelSQL, ModelView):
     _rec_name = 'description'
 
     party = fields.Many2One('party.party', 'Party', required=True, select=1,
-        on_change=['party'], states=_STATES_STOP, depends=['state'])
+            states=_STATES_STOP, depends=_DEPENDS_STOP)
     address = fields.Many2One('party.address', 'Address',
-            domain=[('party', '=', Eval('party'))],
-            select=2, depends=['party', 'state'],
-            states=_STATES_STOP)
+        domain=[('party', '=', Eval('party'))],
+        select=2, depends=['party', 'state'],
+        states=_STATES_STOP)
     company = fields.Many2One('company.company', 'Company', required=True,
-            select=1, states=_STATES_STOP, domain=[
-                ('id', If(In('company', Eval('context', {})), '=', '!='),
-                    Get(Eval('context', {}), 'company', 0)),
-            ], on_change=['company'], depends=['state'])
+        select=1, states=_STATES_STOP, domain=[
+            ('id', If(In('company', Eval('context', {})), '=', '!='),
+                Get(Eval('context', {}), 'company', 0)),
+            ], on_change=['company'], depends=_DEPENDS_STOP)
     currency = fields.Function(fields.Many2One('currency.currency',
         'Currency'), 'get_currency')
     currency_digits = fields.Function(fields.Integer('Currency Digits'),
@@ -50,31 +52,34 @@ class SaleOpportunity(ModelWorkflow, ModelSQL, ModelView):
     amount = fields.Numeric('Amount', digits=(16, Eval('currency_digits', 2)),
             depends=['currency_digits'], help='Estimated revenue amount')
     warehouse = fields.Many2One('stock.location', 'Warehouse',
-            domain=[('type', '=', 'warehouse')], states={
-                'required': Equal(Eval('state'), 'converted'),
-                'readonly': In(Eval('state'),
-                    ['converted', 'lost', 'cancelled']),
-            })
+        domain=[('type', '=', 'warehouse')], states={
+            'required': Eval('state') == 'converted',
+            'readonly': In(Eval('state'),
+                ['converted', 'lost', 'cancelled']),
+            },
+        depends=['state'])
     payment_term = fields.Many2One('account.invoice.payment_term',
-            'Payment Term', states={
-                'required': Equal(Eval('state'), 'converted'),
-                'readonly': In(Eval('state'),
-                    ['converted', 'lost', 'cancelled']),
-            })
+        'Payment Term', states={
+            'required': Eval('state') == 'converted',
+            'readonly': In(Eval('state'),
+                ['converted', 'lost', 'cancelled']),
+            },
+        depends=['state'])
     employee = fields.Many2One('company.employee', 'Employee', required=True,
             states=_STATES_STOP, depends=['state', 'company'],
             domain=[('company', '=', Eval('company'))])
     start_date = fields.Date('Start Date', required=True, select=1,
-            states=_STATES_START, depends=['state'])
+        states=_STATES_START, depends=_DEPENDS_START)
     end_date = fields.Date('End Date', select=2, readonly=True, states={
         'invisible': Not(In(Eval('state'),
             ['converted', 'cancelled', 'lost'])),
     }, depends=['state'])
     description = fields.Char('Description', required=True,
-            states=_STATES_STOP, depends=['state'])
-    comment = fields.Text('Comment', states=_STATES_STOP)
+        states=_STATES_STOP, depends=_DEPENDS_STOP)
+    comment = fields.Text('Comment', states=_STATES_STOP,
+        depends=_DEPENDS_STOP)
     lines = fields.One2Many('sale.opportunity.line', 'opportunity', 'Lines',
-            states=_STATES_STOP, depends=['state'])
+        states=_STATES_STOP, depends=_DEPENDS_STOP)
     state = fields.Selection(STATES, 'State', required=True, select=1,
             sort=False, readonly=True)
     probability = fields.Integer('Conversion Probability',
@@ -84,11 +89,11 @@ class SaleOpportunity(ModelWorkflow, ModelSQL, ModelView):
     history = fields.One2Many('sale.opportunity.history', 'opportunity',
             'History', readonly=True)
     lost_reason = fields.Text('Reason for loss', states={
-        'invisible': Not(Equal(Eval('state'), 'lost')),
-    }, depends=['state'])
+            'invisible': Eval('state') != 'lost',
+            }, depends=['state'])
     sale = fields.Many2One('sale.sale', 'Sale', readonly=True, states={
-        'invisible': Not(Equal(Eval('state'), 'converted')),
-        }, depends=['state'])
+            'invisible': Eval('state') != 'converted',
+            }, depends=['state'])
 
     def __init__(self):
         super(SaleOpportunity, self).__init__()
@@ -106,7 +111,7 @@ class SaleOpportunity(ModelWorkflow, ModelSQL, ModelView):
         return 'lead'
 
     def default_start_date(self):
-        date_obj = self.pool.get('ir.date')
+        date_obj = Pool().get('ir.date')
         return date_obj.today()
 
     def default_probability(self):
@@ -116,8 +121,8 @@ class SaleOpportunity(ModelWorkflow, ModelSQL, ModelView):
         return Transaction().context.get('company') or False
 
     def default_employee(self):
-        user_obj = self.pool.get('res.user')
-        employee_obj = self.pool.get('company.employee')
+        user_obj = Pool().get('res.user')
+        employee_obj = Pool().get('company.employee')
 
         employee_id = False
         if Transaction().context.get('employee'):
@@ -129,14 +134,14 @@ class SaleOpportunity(ModelWorkflow, ModelSQL, ModelView):
         return employee_id
 
     def default_payment_term(self):
-        payment_term_obj = self.pool.get('account.invoice.payment_term')
+        payment_term_obj = Pool().get('account.invoice.payment_term')
         payment_term_ids = payment_term_obj.search(self.payment_term.domain)
         if len(payment_term_ids) == 1:
             return payment_term_ids[0]
         return False
 
     def default_warehouse(self):
-        location_obj = self.pool.get('stock.location')
+        location_obj = Pool().get('stock.location')
         location_ids = location_obj.search(self.warehouse.domain)
         if len(location_ids) == 1:
             return location_ids[0]
@@ -155,7 +160,7 @@ class SaleOpportunity(ModelWorkflow, ModelSQL, ModelView):
         return res
 
     def on_change_company(self, values):
-        company_obj = self.pool.get('company.company')
+        company_obj = Pool().get('company.company')
 
         res = {}
         if values.get('company'):
@@ -166,14 +171,14 @@ class SaleOpportunity(ModelWorkflow, ModelSQL, ModelView):
         return res
 
     def on_change_party(self, values):
-        party_obj = self.pool.get('party.party')
-        payment_term_obj = self.pool.get('account.invoice.payment_term')
+        party_obj = Pool().get('party.party')
+        payment_term_obj = Pool().get('account.invoice.payment_term')
 
         res = {
             'payment_term': False,
         }
-        if values.get('party'):
-            party = party_obj.browse(values['party'])
+        if vals.get('party'):
+            party = party_obj.browse(vals['party'])
             if party.payment_term:
                 res['payment_term'] = party.payment_term.id
         if not res['payment_term']:
@@ -183,19 +188,6 @@ class SaleOpportunity(ModelWorkflow, ModelSQL, ModelView):
                     res['payment_term']).rec_name
         return res
 
-    def set_end_date(self, opportunity_id, state):
-        """
-        This will fill the end_date for a lead/opportunity and change the state
-
-        :param opportunity_id: the id of the opportunity
-        :param state: the target state
-        """
-        date_obj = self.pool.get('ir.date')
-        self.write(opportunity_id, {
-            'end_date': date_obj.today(),
-            'state': state,
-            })
-
     def button_lead(self, ids):
         self.workflow_trigger_create(ids)
         return True
@@ -209,7 +201,7 @@ class SaleOpportunity(ModelWorkflow, ModelSQL, ModelView):
         :return: a dictionary with opportunity line id as key
             and a dictionary of sale line values as value
         '''
-        line_obj = self.pool.get('sale.opportunity.line')
+        line_obj = Pool().get('sale.opportunity.line')
         res = {}
         for line in opportunity.lines:
             val = line_obj.get_sale_line(line)
@@ -244,9 +236,10 @@ class SaleOpportunity(ModelWorkflow, ModelSQL, ModelView):
 
         :param opportunity_id: the id of the opportunity
         '''
-        sale_obj = self.pool.get('sale.sale')
-        sale_line_obj = self.pool.get('sale.line')
-        line_obj = self.pool.get('sale.opportunity.line')
+        pool = Pool()
+        sale_obj = pool.get('sale.sale')
+        sale_line_obj = pool.get('sale.line')
+        line_obj = pool.get('sale.opportunity.line')
 
         opportunity = self.browse(opportunity_id)
 
@@ -269,6 +262,34 @@ class SaleOpportunity(ModelWorkflow, ModelSQL, ModelView):
             })
         return sale_id
 
+    def wkf_lead(self, opportunity):
+        self.write(opportunity.id, {'state': 'lead'})
+
+    def wkf_opportunity(self, opportunity):
+        self.write(opportunity.id, {'state': 'opportunity'})
+
+    def wkf_converted(self, opportunity):
+        date_obj = Pool().get('ir.date')
+        self.write(opportunity.id, {
+                'end_date': date_obj.today(),
+                'state': 'converted',
+                })
+        self.create_sale(opportunity.id)
+
+    def wkf_cancelled(self, opportunity):
+        date_obj = Pool().get('ir.date')
+        self.write(opportunity.id, {
+                'end_date': date_obj.today(),
+                'state': 'cancelled',
+                })
+
+    def wkf_lost(self, opportunity):
+        date_obj = Pool().get('ir.date')
+        self.write(opportunity.id, {
+                'end_date': date_obj.today(),
+                'state': 'lost',
+                })
+
 SaleOpportunity()
 
 
@@ -289,9 +310,9 @@ class SaleOpportunityLine(ModelSQL, ModelView):
     unit_digits = fields.Function(fields.Integer('Unit Digits',
         on_change_with=['unit']), 'get_unit_digits')
     sale_line = fields.Many2One('sale.line', 'Sale Line', readonly=True,
-            states={
-                'invisible': Not(Equal(Get(Eval('_parent_opportunity', {}),
-                        'state'), 'converted')),
+        states={
+            'invisible': (Eval('_parent_opportunity', {}).get( 'state')
+                != 'converted'),
             })
 
     def __init__(self):
@@ -299,7 +320,7 @@ class SaleOpportunityLine(ModelSQL, ModelView):
         self._order.insert(0, ('sequence', 'ASC'))
 
     def on_change_with_unit_digits(self, vals):
-        uom_obj = self.pool.get('product.uom')
+        uom_obj = Pool().get('product.uom')
         if vals.get('unit'):
             uom = uom_obj.browse(vals['unit'])
             return uom.digits
@@ -315,7 +336,7 @@ class SaleOpportunityLine(ModelSQL, ModelView):
         return res
 
     def on_change_product(self, vals):
-        product_obj = self.pool.get('product.product')
+        product_obj = Pool().get('product.product')
 
         if not vals.get('product'):
             return {}
@@ -339,7 +360,7 @@ class SaleOpportunityLine(ModelSQL, ModelView):
         :return: a dictionary with sale line fields as key
             and sale line values as value
         '''
-        sale_line_obj = self.pool.get('sale.line')
+        sale_line_obj = Pool().get('sale.line')
         res = {
             'type': 'line',
             'quantity': line.quantity,
@@ -400,7 +421,7 @@ class SaleOpportunityHistory(ModelSQL, ModelView):
         self._order.insert(0, ('date', 'DESC'))
 
     def _table_query_fields(self):
-        opportunity_obj = self.pool.get('sale.opportunity')
+        opportunity_obj = Pool().get('sale.opportunity')
         table = '%s__history' % opportunity_obj._table
         return [
             'MIN("%s".__id) AS id' % table,
@@ -412,7 +433,7 @@ class SaleOpportunityHistory(ModelSQL, ModelView):
                 and not hasattr(field, 'set')]
 
     def _table_query_group(self):
-        opportunity_obj = self.pool.get('sale.opportunity')
+        opportunity_obj = Pool().get('sale.opportunity')
         table = '%s__history' % opportunity_obj._table
         return [
             '"%s".id' % table,
@@ -423,7 +444,7 @@ class SaleOpportunityHistory(ModelSQL, ModelView):
                 and not hasattr(field, 'set')]
 
     def table_query(self):
-        opportunity_obj = self.pool.get('sale.opportunity')
+        opportunity_obj = Pool().get('sale.opportunity')
         return (('SELECT ' + \
                 (', '.join(self._table_query_fields())) + \
                 ' FROM "%s__history" '
@@ -432,7 +453,7 @@ class SaleOpportunityHistory(ModelSQL, ModelView):
                 opportunity_obj._table, [])
 
     def get_lines(self, ids, name):
-        line_obj = self.pool.get('sale.opportunity.line')
+        line_obj = Pool().get('sale.opportunity.line')
         histories = self.browse(ids)
         result = {}
         # We will always have only one id per call due to datetime_field
@@ -491,7 +512,7 @@ class SaleOpportunityEmployee(ModelSQL, ModelView):
         return ['lost']
 
     def table_query(self):
-        opportunity_obj = self.pool.get('sale.opportunity')
+        opportunity_obj = Pool().get('sale.opportunity')
         clause = ' '
         args = [True]
         if Transaction().context.get('start_date'):
@@ -591,8 +612,8 @@ class OpenSaleOpportunityEmployee(Wizard):
     }
 
     def _action_open(self, data):
-        model_data_obj = self.pool.get('ir.model.data')
-        act_window_obj = self.pool.get('ir.action.act_window')
+        model_data_obj = Pool().get('ir.model.data')
+        act_window_obj = Pool().get('ir.action.act_window')
         act_window_id = model_data_obj.get_id('sale_opportunity',
                 'act_opportunity_employee_form')
         res = act_window_obj.read(act_window_id)
@@ -644,7 +665,7 @@ class SaleOpportunityMonthly(ModelSQL, ModelView):
         return ['lost']
 
     def table_query(self):
-        opportunity_obj = self.pool.get('sale.opportunity')
+        opportunity_obj = Pool().get('sale.opportunity')
         type_id = FIELDS[self.id._type].sql_type(self.id)[0]
         type_year = FIELDS[self.year._type].sql_type(self.year)[0]
         return ('SELECT CAST(id AS ' + type_id + ') AS id, create_uid, '
@@ -755,7 +776,7 @@ class SaleOpportunityEmployeeMonthly(ModelSQL, ModelView):
         return ['lost']
 
     def table_query(self):
-        opportunity_obj = self.pool.get('sale.opportunity')
+        opportunity_obj = Pool().get('sale.opportunity')
         type_id = FIELDS[self.id._type].sql_type(self.id)[0]
         type_year = FIELDS[self.year._type].sql_type(self.year)[0]
         return ('SELECT CAST(id AS ' + type_id + ') AS id, create_uid, '
